#!/bin/sh

###############################################################################
##                SETUP STAPI DEVICE NAME ENVIRONMENT VARIABLES              ##
###############################################################################

# Check if we are on virtual platform (running on x86) or not
# -----------------------------------------------------------
runonvirtualplatform="no"
if [ -f /proc/hce/cwd ]; then
 runonvirtualplatform="yes"
fi

# Verify if root module is specified
# ----------------------------------
export PATH=$PATH:/sbin
if [ -z "$MODULES_INSTALL_DIR" ] ; then
 if [ "$runonvirtualplatform" = "no" ] ; then
  export MODULES_INSTALL_DIR=/root/modules
 else
  export MODULES_INSTALL_DIR=.
 fi
fi
if [ "$runonvirtualplatform" = "no" ] ; then
 source $MODULES_INSTALL_DIR/load_env.sh
else
 . $MODULES_INSTALL_DIR/load_env.sh
fi

if [ -z "$USE_UDEV" ]; then
 dev_root=""
 echo "USE_UDEV:$USE_UDEV"
else
 dev_root="/dev/.static"
 if [ -f ./skip_nodes ] ; then
  cp -a $dev_root/dev/stapi /dev/
  dev_root=""
  echo "skip_nodes"
 fi
fi
export dev_root
echo "dev_root:$dev_root"

# Check if we are in 32 or 29bits
# -------------------------------
if [ -f $MODULES_INSTALL_DIR/load_modules_list_29BITS.txt ]; then
 LOAD_MODULES_LIST=$MODULES_INSTALL_DIR/load_modules_list_29BITS.txt
 MODE="29BITS"
else
 MODE="32BITS"
 if [ -f $MODULES_INSTALL_DIR/load_modules_list_32BITS.txt ]; then
  LOAD_MODULES_LIST=$MODULES_INSTALL_DIR/load_modules_list_32BITS.txt
 else
  LOAD_MODULES_LIST=$MODULES_INSTALL_DIR/load_modules_list.txt
 fi
fi

###############################################################################
##                          SETUP DIRECTORY ACCESS                           ##
###############################################################################

# Verify if modules list exist
# ----------------------------
if [ ! -f $LOAD_MODULES_LIST ] ; then
 echo "$LOAD_MODULES_LIST does not exist"
 exit
fi

# Create /dev/stapi directory if not exist
# ----------------------------------------
if [ -n "$ST_DEV_ROOT_NAME" ] ; then
 if [ ! -e $dev_root/dev/$ST_DEV_ROOT_NAME ]; then mkdir -p $dev_root/dev/$ST_DEV_ROOT_NAME ;fi
 chmod 755 $dev_root/dev/$ST_DEV_ROOT_NAME
fi

###############################################################################
##                          MODULE INSERT PROCEDURE                          ##
###############################################################################

# Return chip version
# -------------------
define_chip_info()
{
 export chipset=`cat /proc/stsys_ioctl/chipset | cut -b1-7`
 export chipver=`cat /proc/stsys_ioctl/chipset | cut -b9-9`
}

# Create a node in /dev/ (Arg 1=node_name)
# ----------------------------------------
create_node()
{
 node_name="$1"
 major=`cat /proc/devices | grep "\b$node_name\b" | cut -b1-3`
 if [ ! -z "$major" ]; then
  dev="$dev_root/dev/$ST_DEV_ROOT_NAME/$node_name"
   if [ -c ${dev} ]; then
    if [ "$runonvirtualplatform" = "no" ] ; then
     cur_major=$((0x`stat -c %t $dev 2>/dev/null`))
    else
     cur_major=`cat /proc/devices | grep $node_name|cut -d " " -f 1`
    fi
    if [ $major -ne $cur_major ]; then
     rm -f ${dev} 2> /dev/null
     echo "mknod -m 0660 $dev c $major 0 || return 1"
     mknod -m 0660 $dev c $major 0 || return 1
    fi
   fi
   if [ ! -c ${dev} ] ; then
    rm -f ${dev} 2> /dev/null
    echo "mknod -m 660 $dev c $major 0 || return 1"
    mknod -m 660 $dev c $major 0 || return 1
   fi
 fi
}

# Load firmware (Arg 1=video1 or audio1)
# --------------------------------------
load_firmware()
{
 if [ "$1" = "video1" ] ; then
  if [ "$chipset" = "STx7105" ] ; then
   cat $MODULES_INSTALL_DIR/companion_7105_video_Ax.bin >$dev_root/dev/"$ST_DEV_ROOT_NAME"/lxload
  fi
 fi
 if [ "$1" = "audio1" ] ; then
  if [ "$chipset" = "STx7105" ] ; then
   cat $MODULES_INSTALL_DIR/companion_7105_audio.bin >$dev_root/dev/"$ST_DEV_ROOT_NAME"/lxload
  fi
 fi
}


# Insert unique STAPI module
# --------------------------
insert_unified_module()
{
 # Inserting sttbx_early_core which provides traces for early loaded .ko
 insmod $MODULES_INSTALL_DIR/sttbx_early_core.ko
 # We are first inserting stsys_ioctl because we need it to know the chip and cut
 echo "* Looking for chipset model and version..."
 insmod $MODULES_INSTALL_DIR/stos_core.ko
 insmod $MODULES_INSTALL_DIR/stsys_ioctl.ko
 define_chip_info
 echo "  $chipset $chipver detected ($MODE)"

 # Now, we load video firmware
 echo "* Loading companions"
 insmod $MODULES_INSTALL_DIR/lxload_sysfs.ko
 insmod $MODULES_INSTALL_DIR/lxload.ko type=video1,audio1
if [ ! -f ./skip_nodes ] ; then
 create_node lxload
fi
 echo "video1" > /sys/kernel/lxloader/lxname
 load_firmware "video1"
 echo "audio1" > /sys/kernel/lxloader/lxname
 load_firmware "audio1"

 # Now we load multicom
 echo "* Inserting Multicom"
 if [ ! -f $MODULES_INSTALL_DIR/ics.ko ] ; then
  insmod $MODULES_INSTALL_DIR/embxshell.ko  || return 1
  insmod $MODULES_INSTALL_DIR/embxloopback.ko  || return 1
  cat $LOAD_MODULES_LIST | grep "embxmailbox\.ko" | grep "$MODE" | while read mode mod_file node_name param; do
  insmod $MODULES_INSTALL_DIR/embxmailbox.ko "$param"; done
  cat $LOAD_MODULES_LIST | grep "embxshm\.ko"  | grep "$MODE"| while read mode mod_file node_name param; do
  insmod $MODULES_INSTALL_DIR/embxshm.ko "$param"; done
  cat $LOAD_MODULES_LIST | grep "mme_host\.ko" | while read mode mod_file node_name param; do
  insmod $MODULES_INSTALL_DIR/mme_host.ko "$param"; done
 else
  cat $LOAD_MODULES_LIST | grep "ics\.ko"  | while read mode mod_file node_name param; do
  if [ "$param" = "-" ] ; then
   param=""
  fi
  insmod $MODULES_INSTALL_DIR/ics.ko "$param"; done
  insmod $MODULES_INSTALL_DIR/ics_user.ko  || return 1
  insmod $MODULES_INSTALL_DIR/embx.ko  || return 1
  insmod $MODULES_INSTALL_DIR/embxmailbox.ko || return 1;
  cat $LOAD_MODULES_LIST | grep "mme\.ko"  | while read mode mod_file node_name param; do
  if [ "$param" = "-" ] ; then
   param=""
  fi
  insmod $MODULES_INSTALL_DIR/mme.ko "$param"; done
  insmod $MODULES_INSTALL_DIR/mme_user.ko || return 1 ;
 fi

 # Now we load STAPI and create nodes
 echo "* Inserting STAPI Module"
 insmod $MODULES_INSTALL_DIR/stapi_core_stripped.ko || return 1
 if [ "$1" != "no_stapi_ioctl" ] ; then
  insmod $MODULES_INSTALL_DIR/stapi_ioctl_stripped.ko || return 1
 else
  if [ -f $MODULES_INSTALL_DIR/stpti4_ioctl.ko ]; then
   insmod $MODULES_INSTALL_DIR/stpti4_ioctl.ko || return 1
  fi
  if [ -f $MODULES_INSTALL_DIR/stpti5_ioctl.ko ]; then
   insmod $MODULES_INSTALL_DIR/stpti5_ioctl.ko || return 1
  fi
 fi
 if [ -f $MODULES_INSTALL_DIR/staudlx_alsa.ko ]; then
  insmod $MODULES_INSTALL_DIR/staudlx_alsa.ko || return 1
 fi
 if [ "$1" != "no_stapi_ioctl" ] ; then
  if [ ! -f ./skip_nodes ] ; then
    echo "* Creating device nodes in /dev/$ST_DEV_ROOT_NAME"
    cat $LOAD_MODULES_LIST | grep -v "^#" | grep -v lxload | (while read mode mod_file node_name param; do
    if [ ! "$node_name" = "-" ] ; then 
     create_node $node_name
    fi
   done)
   echo -n "" > ./skip_nodes
   cp -a $dev_root/dev/stapi /dev/
  fi
 fi
echo " * Load modules OK * "
}

  insert_unified_module $1
